@@ -5,7 +5,9 @@
 """
 
 import argparse
+import sys
 
+sys.path.append('..')
 import pycorrector
 
 
