@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
-# Author: XuMing(xuming624@qq.com)
-# Brief: Corpus for model
+"""
+@author:XuMing(xuming624@qq.com)
+@description:Corpus for model
+"""
 
 import sys
-from codecs import open
+import json
 from collections import Counter
-
+from loguru import logger
 import numpy as np
 
 # Define constants associated with the usual special tokens.
@@ -23,13 +25,16 @@ def save_word_dict(dict_data, save_path):
 
 def load_word_dict(save_path):
     dict_data = dict()
+    num = 0
     with open(save_path, 'r', encoding='utf-8') as f:
         for line in f:
-            items = line.strip().split()
+            line = line.strip('\n')
+            items = line.split('\t')
+            num += 1
             try:
                 dict_data[items[0]] = int(items[1])
             except IndexError:
-                print('error', line)
+                logger.error('IndexError, index:%s, line:%s' % (num, line))
     return dict_data
 
 
@@ -67,7 +72,18 @@ def load_bert_data(path, use_segment, num_examples=None):
     return src_trg_lines
 
 
-def create_dataset(path, num_examples=None):
+class CscDataset(object):
+    def __init__(self, file_path):
+        self.data = json.load(open(file_path, 'r', encoding='utf-8'))
+
+    def load(self):
+        data_list = []
+        for item in self.data:
+            data_list.append(item['original_text'] + '\t' + item['correct_text'])
+        return data_list
+
+
+def create_dataset(path, num_examples=None, split_on_space=False):
     """
     # 1. Remove the accents
     # 2. Clean the sentences
@@ -76,15 +92,19 @@ def create_dataset(path, num_examples=None):
     :param num_examples:
     :return:
     """
-    lines = open(path, 'r', encoding='utf-8').read().strip().split('\n')
-    word_pairs = [[preprocess_sentence(s) for s in l.split('\t')] for l in lines[:num_examples]]
+    if path.endswith('.json'):
+        d = CscDataset(path)
+        lines = d.load()
+    else:
+        lines = open(path, 'r', encoding='utf-8').read().strip().split('\n')
+    word_pairs = [[preprocess_sentence(s, split_on_space) for s in l.split('\t')] for l in lines[:num_examples]]
     return zip(*word_pairs)
 
 
-def preprocess_sentence(sentence):
+def preprocess_sentence(sentence, split_on_space=False):
     # adding a start and an end token to the sentence
     # so that the model know when to start and stop predicting.
-    return [SOS_TOKEN] + sentence.lower().split() + [EOS_TOKEN]
+    return [SOS_TOKEN] + sentence.lower().split() if split_on_space else list(sentence.lower()) + [EOS_TOKEN]
 
 
 def show_progress(curr, total, time=""):
